// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include the std::vector class, which is comparable to the
// ArrayList in Java (this is not a mathematical vector!):
#include <vector>

// Include OpenGL3.3 Core functions:
#include <glad/glad.h>

// Include our Vec4f class of the last exercise:
#include "Vec4f.h"

// Include our Triangle primitive:
#include "Triangle.h"

/**
 * Represents a renderable mesh.
 */
class Mesh
{
    // Pointer to Vertex Array Object (on GPU):
    GLuint vao = 0;

    // Pointer to vertex buffer (interleaved):
    GLuint vbo = 0;
    GLuint vio = 0;

    // Number of vertices:
    unsigned int verticesNum = 0;

    // Store the triangles:
    std::vector<Triangle> triangles;

public:

    /**
     * Constructs a mesh which consists of the given vertices
     * and prepare for rendering (e.g. upload the data to the GPU).
     *
     * You can assume that only triangles are given, where the first
     * triangle is defined by (vertices[0], vertices[1], vertices[2]),
     * the second one by (vertices[3], vertices[4], vertices[5]) etc.
     */
    Mesh(std::vector<Triangle> triangles);

    /**
     * Deletes the mesh and ensures that the memory on the GPU
     * is cleaned up.
     */
    ~Mesh();

    /**
     * Renders the mesh. wf=true for wireframe mode
     */
    void render(bool wf);
};
